# {{ ansible_managed }}
pid_file = "{{ local_temp_dir }}/vault.pid"
{% if cryptography_version.stdout is version('1.6', '>=') %}
listener "tcp" {
  tls_key_file = "{{ local_temp_dir }}/privatekey.pem"
  tls_cert_file = "{{ local_temp_dir }}/cert.pem"
  tls_disable  = false
  address = "localhost:8201"
}
{% endif %}
