plugin: amazon.aws.aws_rds
aws_access_key_id: '{{ aws_access_key }}'
aws_secret_access_key: '{{ aws_secret_key }}'
{% if security_token | default(false) %}
aws_security_token: '{{ security_token }}'
{% endif %}
regions:
  - '{{ aws_region }}'
filters:
  db-instance-id: "{{ resource_prefix }}-mariadb"
