plugin: amazon.aws.aws_ec2
aws_access_key_id: '{{ aws_access_key }}'
aws_secret_access_key: '{{ aws_secret_key }}'
{% if security_token | default(false) %}
aws_security_token: '{{ security_token }}'
{% endif %}
regions:
- '{{ aws_region }}'
filters:
  tag:Name:
  - '{{ resource_prefix }}'
hostnames:
  - name: 'tag:Name'
    separator: '_'
    prefix: 'tag:OtherTag'
