{
    "displayName": "ansible_test",
    "name": "ansible_test",
    "description": "",
    "siteAssociations": [],
    "userAssociations": [{
    "userId": "{{ query_user_id.current.id }}"
    }],
    "_updateVersion": 0,
}