{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Sid": "AssumeFromCloudTrails",
      "Effect": "Allow",
      "Principal": {
        "Service": "cloudtrail.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}
