apiVersion: v1
kind: Pod
metadata:
  labels:
    app: '[[ k8s_pod_name ]]'
  name: '[[ k8s_pod_name ]]'
  namespace: '[[ k8s_pod_namespace ]]'
spec:
  containers:
  - args:
    - /bin/sh
    - -c
    - while true; do echo $(date); sleep 10; done
    image: python:3.7-alpine
    imagePullPolicy: Always
    name: '[[ k8s_pod_name ]]'
