prefix-set EBGP-PEER-BOGONS
  192.0.2.0/24 le 32,
  192.168.0.0/16 le 32,
  198.18.0.0/16 le 32,
  224.0.0.0/4 le 32,
  240.0.0.0/4 le 32
end-set
!
as-path-set EBGP-PEER-AS16509-403-PERMIT-PATHS
  ios-regex '^11164_8075_',
  ios-regex '^1164_16509$',
  ios-regex '^1116_16409_[0-9]+$',
  ios-regex '^8075_'
end-set
!
community-set cust-announce
  1525:65298,
  1525:6546,
  1525:6438,
  1525:65439,
  1525:65498
end-set
!
rd-set EBGP_INCOMING_RD_SET
  172.16.0.0/16:*,
  172.14.0.0/16:100,
  192:*,
  192:100
end-set
!
extcommunity-set rt EBGP_INCOMIG_RT_SET
  10:615,
  10:6120,
  15.15.15.15:15
end-set
!
route-policy POLICY2
end-policy
!
route-policy static-to-bgp
  if destination in cust-no-export then
    apply cust2bgp
    set community cust-no-export additive
  elseif destination in cust-announce then
    apply cust2bgp
    set community cust-announce additive
  elseif destination in cust-announce-backup then
    apply cust2bgp
    set local-preference 100
    set weight 23
    set community cust-announce additive
  elseif destination in cust-no-export-backup then
    apply cust2bgp
    set local-preference 98
    set weight 0
    set community cust-no-export additive
  else
    drop
  endif
end-policy
!
class-map match-any data
 match precedence ipv4 0 1 2
 end-class-map
!
